@@ -1,15 +1,19 @@
 import time
-from sklearn.metrics import r2_score, mean_squared_error, mean_absolute_error
-import pandas as pd
+
 import numpy as np
+import pandas as pd
+from IPython.display import display
+from luciferml.supervised.utils.configs import regressors
+from luciferml.supervised.utils.dimensionalityReduction import \
+    dimensionalityReduction
 from luciferml.supervised.utils.encoder import encoder
-from luciferml.supervised.utils.predPreprocess import pred_preprocess
-from luciferml.supervised.utils.dimensionalityReduction import dimensionalityReduction
-from luciferml.supervised.utils.regressionPredictor import regressionPredictor
-from luciferml.supervised.utils.kfold import kfold
 from luciferml.supervised.utils.hyperTune import hyperTune
-from luciferml.supervised.utils.sparseCheck import sparseCheck
 from luciferml.supervised.utils.intro import intro
+from luciferml.supervised.utils.kfold import kfold
+from luciferml.supervised.utils.predPreprocess import pred_preprocess
+from luciferml.supervised.utils.regressionPredictor import regressionPredictor
+from luciferml.supervised.utils.sparseCheck import sparseCheck
+from sklearn.metrics import mean_absolute_error, mean_squared_error, r2_score
 
 
 class Regression:
@@ -37,7 +41,8 @@ class Regression:
                  tune_mode=1,
                  smote='n',
                  k_neighbors=1,
-                 dropout_rate = 0
+                 dropout_rate=0,
+                 verbose=False
                  ):
         """
         Encodes Categorical Data then Applies SMOTE , Splits the features and labels in training and validation sets with test_size = .2
@@ -59,34 +64,21 @@ class Regression:
                     Predicting model to be used
                     Default 'lin'
                         Predictor Strings:
-
                                 lin  - Linear Regression
-
                                 sgd  - Stochastic Gradient Descent Regressor
-
-                                elas - Elastic Net Regressot
-
+                                elas - Elastic Net Regressor
                                 krr  - Kernel Ridge Regressor
-
                                 br   - Bayesian Ridge Regressor
-
                                 svr  - Support Vector Regressor
-
                                 knr  - K-Nearest Regressor
-
                                 dt   - Decision Trees
-
                                 rfr  - Random Forest Regressor
-
                                 gbr  - Gradient Boost Regressor
-
                                 lgbm - LightGB Regressor
-
                                 xgb  - XGBoost Regressor
-
                                 cat  - Catboost Regressor
-
                                 ann  - Artificial Neural Network
+                                all  - Applies all above regressors
             params : dict
                         contains parameters for model
             tune : boolean
@@ -141,9 +133,11 @@ class Regression:
                 Whether to apply SMOTE. Default = 'y'
             k_neighbors : int
                 No. of neighbours for SMOTE. Default = 1
+            verbose : boolean
+                Verbosity of models. Default = False
         Returns:
-
             Dict Containing Name of Regressor, Its K-Fold Cross Validated Accuracy, RMSE, Prediction set
+            Dataframe containing all the models and their accuracies when predictor is 'all'
 
         Example:
 
@@ -188,6 +182,7 @@ class Regression:
         self.rerun = False
         self.smote = smote
         self.k_neighbors = k_neighbors
+        self.verbose = verbose
 
         self.accuracy_scores = {}
         self.reg_result = {}
@@ -198,6 +193,14 @@ class Regression:
         self.regressor_name = ''
         self.sc = 0
 
+        self.kfoldacc = []
+        self.acc = []
+        self.mae = []
+        self.rmse = []
+        self.bestacc = []
+        self.bestparams = []
+        self.result_df = pd.DataFrame(index=regressors.values())
+
     def fit(self, features, labels):
         """[Takes Features and Labels and Encodes Categorical Data then Applies SMOTE , Splits the features and labels in training and validation sets with test_size = .2
         scales X_train, X_val using StandardScaler.
@@ -216,7 +219,7 @@ class Regression:
 
         # Time Function ---------------------------------------------------------------------
 
-        start = time.time()
+        self.start = time.time()
         intro()
         print("Started Lucifer-ML \n")
         if not self.rerun:
@@ -224,7 +227,7 @@ class Regression:
             if not isinstance(self.features, pd.DataFrame) and not isinstance(self.labels, pd.Series):
                 print('TypeError: This Function take features as Pandas Dataframe and labels as Pandas Series. Please check your implementation.\n')
                 end = time.time()
-                print(end - start)
+                print(self.end - self.start)
                 return
 
             # Encoding ---------------------------------------------------------------------
@@ -236,26 +239,31 @@ class Regression:
                 self.features, self.labels)
 
             # Preprocessing ---------------------------------------------------------------------
-            self.X_train, X_val, self.y_train, y_val, self.sc = pred_preprocess(
+            self.X_train, self.X_val, self.y_train, self.y_val, self.sc = pred_preprocess(
                 self.features, self.labels, self.test_size, self.random_state, self.smote, self.k_neighbors)
 
             # Dimensionality Reduction---------------------------------------------------------------------
-            self.X_train, X_val = dimensionalityReduction(
-                self.lda, self.pca, self.X_train, X_val, self.y_train,
-                self.n_components_lda, self.n_components_pca, self.pca_kernel, start)
+            self.X_train, self.X_val = dimensionalityReduction(
+                self.lda, self.pca, self.X_train, self.X_val, self.y_train,
+                self.n_components_lda, self.n_components_pca, self.pca_kernel, self.start)
 
         # Models ---------------------------------------------------------------------
+        if self.predictor == 'all':
+            self.pred_mode = 'all'
+            self._fitall()
+            return self.result_df
         if self.predictor == 'ann':
             self.parameters, self.regressor, self.regressor_wrap = regressionPredictor(
-                self.predictor, self.params, self.X_train, X_val, self.y_train, y_val, self.epochs, self.hidden_layers,
+                self.predictor, self.params, self.X_train, self.X_val, self.y_train, self.y_val, self.epochs, self.hidden_layers,
                 self.input_activation, self.loss,
-                self.batch_size, self.validation_split, self.optimizer, self.output_units, self.input_units, self.tune_mode,self.dropout_rate
+                self.batch_size, self.validation_split, self.optimizer, self.output_units, self.input_units, self.tune_mode, self.dropout_rate, verbose=self.verbose
             )
         else:
             self.parameters, self.regressor = regressionPredictor(
-                self.predictor, self.params, self.X_train, X_val, self.y_train, y_val, self.epochs, self.hidden_layers,
+                self.predictor, self.params, self.X_train, self.X_val, self.y_train, self.y_val, self.epochs, self.hidden_layers,
                 self.input_activation,  self.loss,
-                self.batch_size, self.validation_split, self.optimizer, self.output_units, self.input_units, self.tune_mode
+                self.batch_size, self.validation_split, self.optimizer, self.output_units, self.input_units, self.tune_mode,
+                verbose=self.verbose
             )
 
         try:
@@ -268,7 +276,7 @@ class Regression:
         print('Model Training Done [', u'\u2713', ']\n')
         print('Predicting Data [*]\n')
         try:
-            self.y_pred = self.regressor.predict(X_val)
+            self.y_pred = self.regressor.predict(self.X_val)
             print('Data Prediction Done [', u'\u2713', ']\n')
         except Exception as error:
             print('Prediction Failed with error: ', error,  '\n')
@@ -276,10 +284,11 @@ class Regression:
         # Accuracy ---------------------------------------------------------------------
         print('''Evaluating Model Performance [*]''')
         try:
-            self.accuracy = r2_score(y_val, self.y_pred)
-            self.m_absolute_error = mean_absolute_error(y_val, self.y_pred)
+            self.accuracy = r2_score(self.y_val, self.y_pred)
+            self.m_absolute_error = mean_absolute_error(
+                self.y_val, self.y_pred)
             self.rm_squared_error = mean_squared_error(
-                y_val, self.y_pred, squared=False)
+                self.y_val, self.y_pred, squared=False)
             print('Validation R2 Score is {:.2f} %'.format(self.accuracy*100))
             print('Validation Mean Absolute Error is :',
                   self.m_absolute_error)
@@ -293,15 +302,13 @@ class Regression:
         if self.predictor == 'ann':
             self.regressor_name, self.kfold_accuracy = kfold(
                 self.regressor_wrap,
-                self.predictor, self.X_train, self.y_train, self.cv_folds, True
-
-
+                self.predictor, self.X_train, self.y_train, self.cv_folds, isReg=True
             )
         else:
             self.regressor_name, self.kfold_accuracy = kfold(
                 self.regressor,
                 self.predictor,
-                self.X_train, self.y_train, self.cv_folds, True
+                self.X_train, self.y_train, self.cv_folds, isReg=True
             )
 
         # GridSearch ---------------------------------------------------------------------
@@ -309,38 +316,120 @@ class Regression:
             self.__tuner()
 
         print('Complete [', u'\u2713', ']\n')
-        end = time.time()
-        print('Time Elapsed : ', end - start, 'seconds \n')
+        self.end = time.time()
+        print('Time Elapsed : ', self.end - self.start, 'seconds \n')
 
-    def __tuner(self):
+    def _fitall(self):
+        print('Training All Regressors [*]')
+        for index, self.predictor in enumerate(regressors):
+            if not self.predictor == 'ann':
+                self.parameters, self.regressor = regressionPredictor(
+                    self.predictor, self.params, self.X_train, self.X_val, self.y_train, self.y_val, self.epochs, self.hidden_layers,
+                    self.input_activation, self.loss,
+                    self.batch_size, self.validation_split, self.optimizer, self.output_units, self.input_units, self.tune_mode, all_mode=True, verbose=self.verbose
+                )
+            elif self.predictor == 'ann':
+                self.parameters, self.regressor, self.regressor_wrap = regressionPredictor(
+                    self.predictor, self.params, self.X_train, self.X_val, self.y_train, self.y_val, self.epochs, self.hidden_layers,
+                    self.input_activation, self.loss,
+                    self.batch_size, self.validation_split, self.optimizer,
+                    self.output_units, self.input_units, self.tune_mode, self.dropout_rate, all_mode=True, verbose=self.verbose
+                )
+            try:
+
+                if not self.predictor == 'ann':
+                    self.regressor.fit(self.X_train, self.y_train)
+            except Exception as error:
+                print(regressors[self.predictor],
+                      'Model Train Failed with error: ', error, '\n')
+            try:
+                self.y_pred = self.regressor.predict(self.X_val)
+            except Exception as error:
+                print(regressors[self.predictor],
+                      'Data Prediction Failed with error: ', error,  '\n')
+            if self.predictor == 'ann':
+                self.y_pred = (self.y_pred > 0.5).astype("int32")
+
+            # Accuracy ---------------------------------------------------------------------
+            try:
+                self.accuracy = r2_score(self.y_val, self.y_pred)
+                self.m_absolute_error = mean_absolute_error(
+                    self.y_val, self.y_pred)
+                self.rm_squared_error = mean_squared_error(
+                    self.y_val, self.y_pred, squared=False)
+                self.acc.append(self.accuracy*100)
+                self.rmse.append(self.rm_squared_error)
+                self.mae.append(self.m_absolute_error)
+            except Exception as error:
+                print(regressors[self.predictor],
+                      'Evaluation Failed with error: ', error, '\n')
+
+            # K-Fold ---------------------------------------------------------------------
+            if self.predictor == 'ann':
+                self.regressor_name, self.kfold_accuracy = kfold(
+                    self.regressor_wrap,
+                    self.predictor, self.X_train, self.y_train, self.cv_folds, all_mode=True, isReg=True
+                )
+            else:
+                self.regressor_name, self.kfold_accuracy = kfold(
+                    self.regressor,
+                    self.predictor,
+                    self.X_train, self.y_train, self.cv_folds, all_mode=True, isReg=True
+                )
+            self.kfoldacc.append(self.kfold_accuracy)
+            # GridSearch ---------------------------------------------------------------------
+            if not self.predictor == 'nb' and self.tune:
+                self.__tuner(all_mode=True)
+            if self.predictor == 'nb':
+                self.best_params = ''
+                self.best_accuracy = self.kfold_accuracy
+        self.result_df['R2 Score'] = self.acc
+        self.result_df['Mean Absolute Error'] = self.mae
+        self.result_df['Root Mean Squared Error'] = self.rmse
+        self.result_df['KFold Accuracy'] = self.kfoldacc
+        if self.tune:
+            self.result_df['Best Parameters'] = self.bestparams
+            self.result_df['Best Accuracy'] = self.bestacc
+
+        display(self.result_df)
+        print('Complete [', u'\u2713', ']\n')
+        self.end = time.time()
+        print('Time Elapsed : ', self.end - self.start, 'seconds \n')
+
+    def __tuner(self, all_mode=False):
+        if not all_mode:
+            print(
+                'Applying Grid Search Cross validation on Mode {} [*]'.format(self.tune_mode))
         if self.predictor == 'ann':
-            self.best_params = hyperTune(
-                self.regressor_wrap, self.parameters, self.X_train, self.y_train, self.cv_folds, self.tune_mode, True)
+            self.best_params, self.best_accuracy = hyperTune(
+                self.regressor_wrap, self.parameters, self.X_train, self.y_train, self.cv_folds, self.tune_mode, all_mode=all_mode, isReg=True)
         else:
-            self.best_params = hyperTune(
-                self.regressor, self.parameters, self.X_train, self.y_train, self.cv_folds, self.tune_mode, True)
+            self.best_params, self.best_accuracy = hyperTune(
+                self.regressor, self.parameters, self.X_train, self.y_train, self.cv_folds, self.tune_mode, all_mode=all_mode, isReg=True)
+        self.bestparams.append(self.best_params)
+        self.bestacc.append(self.best_accuracy*100)
 
     def result(self):
         """[Makes a dictionary containing Regressor Name, K-Fold CV Accuracy, RMSE, Prediction set.]
 
         Returns:
             [dict]: [Dictionary containing :
-
                         - "Regressor" - Regressor Name
-
                         - "Accuracy" - KFold CV Accuracy
-
                         - "RMSE" - Root Mean Square
-
                         - "YPred" - Array for Prediction set
                         ]
         """
-        self.reg_result['Regressor'] = self.regressor_name
-        self.reg_result['Accuracy'] = self.kfold_accuracy
-        self.reg_result['RMSE'] = self.rm_squared_error
-        self.reg_result['YPred'] = self.y_pred
+        if not self.pred_mode == 'all':
+            self.reg_result['Regressor'] = self.regressor_name
+            self.reg_result['Accuracy'] = self.kfold_accuracy
+            self.reg_result['RMSE'] = self.rm_squared_error
+            self.reg_result['YPred'] = self.y_pred
 
-        return self.reg_result
+            return self.reg_result
+        if self.pred_mode == 'all':
+            print('[Error] This method is only applicable on single predictor')
+            return
 
     def predict(self, X_test):
         """[Takes test set and returns predictions for that test set]
@@ -351,12 +440,16 @@ class Regression:
         Returns:
             [Array]: [Predicted set for given test set]
         """
-        X_test = np.array(X_test)
-        if X_test.ndim == 1:
-            print('''Input Array has only 1-Dimension but 2-Dimension was expected. 
-                  Reshaping it to 2-Dimension''')
-            X_test = X_test.reshape(1, -1)
+        if not self.pred_mode == 'all':
+            X_test = np.array(X_test)
+            if X_test.ndim == 1:
+                print('''Input Array has only 1-Dimension but 2-Dimension was expected. 
+                    Reshaping it to 2-Dimension''')
+                X_test = X_test.reshape(1, -1)
 
-        y_test = self.regressor.predict(self.sc.transform(X_test))
+            y_test = self.regressor.predict(self.sc.transform(X_test))
 
-        return y_test
+            return y_test
+        if self.pred_mode == 'all':
+            print('[Error] This method is only applicable on single predictor')
+            return
