@@ -1,30 +1,30 @@
+import tensorflow as tf
+from catboost import CatBoostRegressor
+from lightgbm import LGBMRegressor
+from luciferml.supervised.utils.regression_params import *
+from sklearn.ensemble import GradientBoostingRegressor, RandomForestRegressor
+from sklearn.kernel_ridge import KernelRidge
+from sklearn.linear_model import (BayesianRidge, ElasticNet, LinearRegression,
+                                  SGDRegressor)
 from sklearn.neighbors import KNeighborsRegressor
 from sklearn.svm import SVR
 from sklearn.tree import DecisionTreeRegressor
 from xgboost import XGBRegressor
-from sklearn.ensemble import RandomForestRegressor, GradientBoostingRegressor
-from sklearn.linear_model import LinearRegression, SGDRegressor, ElasticNet, BayesianRidge
-from lightgbm import LGBMRegressor
-from catboost import CatBoostRegressor
-from sklearn.kernel_ridge import KernelRidge
-
-import tensorflow as tf
-from luciferml.supervised.utils.regression_params import *
 
 
 def regressionPredictor(
         predictor, params, X_train, X_val, y_train, y_val, epochs, hidden_layers,
         input_activation, loss,
-        batch_size, validation_split, optimizer, output_units, input_units, 
-        tune_mode,dropout_rate=0
+        batch_size, validation_split, optimizer, output_units, input_units,
+        tune_mode, dropout_rate=0, all_mode=False, verbose=False
 ):
     """
     Takes Predictor string , parameters , Training and Validation set and Returns a regressor for the Choosen Predictor.
     """
     try:
         if predictor == 'lin':
-            print('Training Logistic Regression on Training Set [*]\n')
-
+            if not all_mode:
+                print('Training Logistic Regression on Training Set [*]\n')
             regressor = LinearRegression(**params)
             if tune_mode == 1:
                 parameters = parameters_lin
@@ -34,9 +34,9 @@ def regressionPredictor(
                 parameters = parameters_lin
 
         elif predictor == 'sgd':
-            print(
-                'Training Stochastic Gradient Descent Regressor on Training Set [*]\n')
-
+            if not all_mode:
+                print(
+                    'Training Stochastic Gradient Descent Regressor on Training Set [*]\n')
             regressor = SGDRegressor(**params)
             if tune_mode == 1:
                 parameters = parameters_sgd_1
@@ -46,8 +46,8 @@ def regressionPredictor(
                 parameters = parameters_sgd_3
 
         elif predictor == 'krr':
-            print('Training Kernel Ridge Regressor on Training Set [*]\n')
-
+            if not all_mode:
+                print('Training Kernel Ridge Regressor on Training Set [*]\n')
             regressor = KernelRidge(**params)
             if tune_mode == 1:
                 parameters = parameters_ker_1
@@ -55,9 +55,10 @@ def regressionPredictor(
                 parameters = parameters_ker_2
             elif tune_mode == 3:
                 parameters = parameters_ker_3
-        elif predictor == 'elas':
-            print('Training ElasticNet Regressor on Training Set [*]\n')
 
+        elif predictor == 'elas':
+            if not all_mode:
+                print('Training ElasticNet Regressor on Training Set [*]\n')
             regressor = ElasticNet(**params)
             if tune_mode == 1:
                 parameters = parameters_elas
@@ -67,8 +68,8 @@ def regressionPredictor(
                 parameters = parameters_elas
 
         elif predictor == 'br':
-            print('Training BayesianRidge Regressor on Training Set [*]\n')
-
+            if not all_mode:
+                print('Training BayesianRidge Regressor on Training Set [*]\n')
             regressor = BayesianRidge(**params)
             if tune_mode == 1:
                 parameters = parameters_br
@@ -78,8 +79,8 @@ def regressionPredictor(
                 parameters = parameters_br
 
         elif predictor == 'svr':
-            print('Training Support Vector Machine on Training Set [*]\n')
-
+            if not all_mode:
+                print('Training Support Vector Machine on Training Set [*]\n')
             regressor = SVR(**params)
             if tune_mode == 1:
                 parameters = parameters_svr_1
@@ -89,7 +90,8 @@ def regressionPredictor(
                 parameters = parameters_svr_3
 
         elif predictor == 'knr':
-            print('Training KNeighbors Regressor on Training Set [*]\n')
+            if not all_mode:
+                print('Training KNeighbors Regressor on Training Set [*]\n')
             regressor = KNeighborsRegressor(**params)
             if tune_mode == 1:
                 parameters = parameters_knr_1
@@ -99,7 +101,8 @@ def regressionPredictor(
                 parameters = parameters_knr_3
 
         elif predictor == 'dt':
-            print('Training Decision Tree regressor on Training Set [*]\n')
+            if not all_mode:
+                print('Training Decision Tree regressor on Training Set [*]\n')
             regressor = DecisionTreeRegressor(**params)
             if tune_mode == 1:
                 parameters = parameters_dt_1
@@ -109,7 +112,8 @@ def regressionPredictor(
                 parameters = parameters_dt_3
 
         elif predictor == 'rfr':
-            print('Training Random Forest regressor on Training Set [*]\n')
+            if not all_mode:
+                print('Training Random Forest regressor on Training Set [*]\n')
             regressor = RandomForestRegressor(**params)
             if tune_mode == 1:
                 parameters = parameters_rfr_1
@@ -119,8 +123,9 @@ def regressionPredictor(
                 parameters = parameters_rfr_3
 
         elif predictor == 'gbr':
-            print(
-                'Training Gradient Boosting Regressor  on Training Set [*]\n')
+            if not all_mode:
+                print(
+                    'Training Gradient Boosting Regressor  on Training Set [*]\n')
             regressor = GradientBoostingRegressor(**params)
             if tune_mode == 1:
                 parameters = parameters_gbr_1
@@ -130,7 +135,8 @@ def regressionPredictor(
                 parameters = parameters_gbr_3
 
         elif predictor == 'xgb':
-            print('Training XGBregressor on Training Set [*]\n')
+            if not all_mode:
+                print('Training XGBregressor on Training Set [*]\n')
             regressor = XGBRegressor(**params)
             if tune_mode == 1:
                 parameters = parameters_xgb_1
@@ -140,7 +146,8 @@ def regressionPredictor(
                 parameters = parameters_xgb_3
 
         elif predictor == 'lgbm':
-            print('Training LGBMRegressor on Training Set [*]\n')
+            if not all_mode:
+                print('Training LGBMRegressor on Training Set [*]\n')
             regressor = LGBMRegressor(**params)
             if tune_mode == 1:
                 parameters = parameters_lgbm_1
@@ -150,7 +157,9 @@ def regressionPredictor(
                 parameters = parameters_lgbm_3
 
         elif predictor == 'cat':
-            print('Training CatBoost Regressor on Training Set [*]\n')
+            if not all_mode:
+                print('Training CatBoost Regressor on Training Set [*]\n')
+            params['verbose'] = verbose
             regressor = CatBoostRegressor(**params)
             if tune_mode == 1:
                 parameters = parameters_cat
@@ -160,13 +169,13 @@ def regressionPredictor(
                 parameters = parameters_cat
 
         elif predictor == 'ann':
-            def build_ann_model(input_units,optimizer,rate):
+            def build_ann_model(input_units, optimizer, rate):
                 try:
                     regressor = tf.keras.models.Sequential()
                     for i in range(0, hidden_layers):
                         regressor.add(tf.keras.layers.Dense(
                             units=input_units, activation=input_activation))
-                        regressor.add(tf.keras.layers.Dropout(rate = rate))
+                        regressor.add(tf.keras.layers.Dropout(rate=rate))
                     regressor.add(tf.keras.layers.Dense(
                         units=output_units))
                     regressor.compile(optimizer=optimizer,
@@ -175,17 +184,22 @@ def regressionPredictor(
 
                 except Exception as error:
                     print('ANN Build Failed with error :', error, '\n')
-            print('Training ANN on Training Set [*]\n')
-            regressor = build_ann_model(input_units, optimizer,dropout_rate)
+            if not all_mode:
+                print('Training ANN on Training Set [*]\n')
+            regressor = build_ann_model(input_units, optimizer, dropout_rate)
 
+            if verbose == False:
+                verbose = 0
+            if verbose == True:
+                verbose = 'auto'
             ann_history = regressor.fit(
                 X_train, y_train, validation_split=validation_split,
                 validation_data=(
-                    X_val, y_val), epochs=epochs, batch_size=batch_size
+                    X_val, y_val), epochs=epochs, batch_size=batch_size, verbose=verbose
             )
             regressor_wrap = tf.keras.wrappers.scikit_learn.KerasRegressor(
-                build_fn=build_ann_model, verbose=1, input_units=input_units,
-                epochs=epochs, batch_size=batch_size, optimizer=optimizer, rate = dropout_rate
+                build_fn=build_ann_model, input_units=input_units,
+                epochs=epochs, batch_size=batch_size, optimizer=optimizer, rate=dropout_rate
             )
             if tune_mode == 1:
                 parameters = parameters_ann_1
